# This file was auto-generated by Fern from our API Definition.

import typing

EntityType = typing.Union[
    typing.Literal[
        "CONVERSATION",
        "CONVERSATION_MESSAGE",
        "KNOWLEDGE_BASE",
        "KNOWLEDGE_DOCUMENT",
        "ACTION",
        "USER",
        "EVENT",
        "EVENT_TRIGGER",
        "USER_PROFILE",
        "FEEDBACK",
    ],
    typing.Any,
]
