# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ActionEnumOption(UniversalBaseModel):
    label: str = pydantic.Field()
    """
    Will be shown during form display as the user facing string in a dropdown or radio.
    """

    value: typing.Optional[typing.Any] = pydantic.Field(default=None)
    """
    A valid value for the field.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
