# This file was auto-generated by Fern from our API Definition.

from .event_trigger_base import EventTriggerBase
import typing_extensions
from .entity_id import EntityId
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class EventTriggerResponse(EventTriggerBase):
    """
    Examples
    --------
    from mavenagi.commons import EntityId, EventTriggerResponse

    EventTriggerResponse(
        trigger_id=EntityId(
            reference_id="store-in-snowflake",
            app_id="snowflake",
            organization_id="acme",
            agent_id="support",
            type="EVENT_TRIGGER",
        ),
        description="Stores conversation data in Snowflake",
        type="CONVERSATION_CREATED",
    )
    """

    trigger_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="triggerId")] = pydantic.Field()
    """
    ID that uniquely identifies this event trigger
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
